/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizappointmentrecord.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fhs.core.trans.anno.Trans;
import com.fhs.core.trans.constant.TransType;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Getter;
import lombok.Setter;
import vip.xiaonuo.common.pojo.CommonEntity;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 预约记录实体
 *
 * @author fanzherong
 * @date  2025/03/24 14:47
 **/
@Getter
@Setter
@TableName("biz_appointment_record")
public class BizAppointmentRecord extends CommonEntity {

    /** 主键ID */
    @TableId
    @Schema(description = "主键ID")
    private String id;

    /** 订单id */
    @Schema(description = "订单id")
    private String orderId;

    /** 车牌号 */
    @Schema(description = "车牌号")
    private String licenseNumber;

    /** 预约时段id */
    @Schema(description = "预约时段id")
    private String timeId;

    /** 司机姓名 */
    @Schema(description = "司机姓名")
    private String driverName;

    /** 司机电话 */
    @Schema(description = "司机电话")
    private String driverMobile;

    /** 状态（1：待审核  2：审核不通过   3：排队中  4：可入场  5：已入场  6：一次过磅  7：完成装货   8：二次过磅   9：待出场  10：已出场   11：已签收   12:自动审核   13：销售已审核   14：已取消） */
    @Trans(type = TransType.DICTIONARY, key = "appointment_status")
    private String status;

    /**审核备注*/
    private String appointmentReason;

    /**订单编号*/
    @TableField(exist = false)
    private String orderNumber;

    /**订单名称*/
    @TableField(exist = false)
    private String orderName;

    /**客户名称*/
    @TableField(exist = false)
    private String customerName;

    /**货品名称*/
    @TableField(exist = false)
    private String goodsName;

    /**货品规格*/
    @TableField(exist = false)
    private String goodsModel;

    /**开始时间段*/
    @TableField(exist = false)
    private String beginTime;

    /**结束时间段*/
    @TableField(exist = false)
    private String endTime;

    /**标识*/
    @TableField(exist = false)
    private String isFlag;

    /**超限配置id*/
    private String overId;

    /**订单余量*/
    @TableField(exist = false)
    private BigDecimal orderBalance;

    /**超限重量*/
    @TableField(exist = false)
    private BigDecimal overWeight;

    /**订单重量校验*/
    @TableField(exist = false)
    private String orderWeightSwitch;

    @TableField(exist = false)
    /**车辆轴数*/
    private Integer axleNumber;

    /**过磅记录id*/
    @TableField(exist = false)
    private String recordId;

    /**排队号*/
    private Integer queueNumber;

    @TableField(exist = false)
    /**司机签名*/
    private String driverSign;

    @TableField(exist = false)
    /**订单类型*/
    private String orderType;

    @TableField(exist = false)
    /**卸货重量*/
    private BigDecimal unloadWeight;

    /**单据图片*/
    @TableField(exist = false)
    private String unloadImg;
    @TableField(exist = false)
    private String unloadName;

    @TableField(exist = false)
    private String customerContactName;
}
