/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.oawxsendlog.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import jakarta.annotation.Resource;
import jakarta.validation.constraints.NotEmpty;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.oawxsendlog.entity.OaWxSendLog;
import vip.xiaonuo.biz.modular.oawxsendlog.param.OaWxSendLogAddParam;
import vip.xiaonuo.biz.modular.oawxsendlog.param.OaWxSendLogEditParam;
import vip.xiaonuo.biz.modular.oawxsendlog.param.OaWxSendLogIdParam;
import vip.xiaonuo.biz.modular.oawxsendlog.param.OaWxSendLogPageParam;
import vip.xiaonuo.biz.modular.oawxsendlog.service.OaWxSendLogService;

import javax.validation.Valid;
import java.util.List;

/**
 * 微信消息推送日志控制器
 *
 * @author xiaofan
 * @date  2023/12/18 15:24
 */
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class OaWxSendLogController {

    @Resource
    private OaWxSendLogService oaWxSendLogService;

    /**
     * 获取微信消息推送日志分页
     *
     * @author xiaofan
     * @date  2023/12/18 15:24
     */
    @ApiOperationSupport(order = 1)
    @SaCheckPermission("/biz/oawxsendlog/page")
    @GetMapping("/biz/oawxsendlog/page")
    public CommonResult<Page<OaWxSendLog>> page(OaWxSendLogPageParam oaWxSendLogPageParam) {
        return CommonResult.data(oaWxSendLogService.page(oaWxSendLogPageParam));
    }

    /**
     * 添加微信消息推送日志
     *
     * @author xiaofan
     * @date  2023/12/18 15:24
     */
    @ApiOperationSupport(order = 2)
    @CommonLog("添加微信消息推送日志")
    @SaCheckPermission("/biz/oawxsendlog/add")
    @PostMapping("/biz/oawxsendlog/add")
    public CommonResult<String> add(@RequestBody @Valid OaWxSendLogAddParam oaWxSendLogAddParam) {
        oaWxSendLogService.add(oaWxSendLogAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑微信消息推送日志
     *
     * @author xiaofan
     * @date  2023/12/18 15:24
     */
    @ApiOperationSupport(order = 3)
    @CommonLog("编辑微信消息推送日志")
    @SaCheckPermission("/biz/oawxsendlog/edit")
    @PostMapping("/biz/oawxsendlog/edit")
    public CommonResult<String> edit(@RequestBody @Valid OaWxSendLogEditParam oaWxSendLogEditParam) {
        oaWxSendLogService.edit(oaWxSendLogEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除微信消息推送日志
     *
     * @author xiaofan
     * @date  2023/12/18 15:24
     */
    @ApiOperationSupport(order = 4)
    @CommonLog("删除微信消息推送日志")
    @SaCheckPermission("/biz/oawxsendlog/delete")
    @PostMapping("/biz/oawxsendlog/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                       List<OaWxSendLogIdParam> oaWxSendLogIdParamList) {
        oaWxSendLogService.delete(oaWxSendLogIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取微信消息推送日志详情
     *
     * @author xiaofan
     * @date  2023/12/18 15:24
     */
    @ApiOperationSupport(order = 5)
    @SaCheckPermission("/biz/oawxsendlog/detail")
    @GetMapping("/biz/oawxsendlog/detail")
    public CommonResult<OaWxSendLog> detail(@Valid OaWxSendLogIdParam oaWxSendLogIdParam) {
        return CommonResult.data(oaWxSendLogService.detail(oaWxSendLogIdParam));
    }
}
