/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.oawxsendlog.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.oawxsendlog.entity.OaWxSendLog;
import vip.xiaonuo.biz.modular.oawxsendlog.mapper.OaWxSendLogMapper;
import vip.xiaonuo.biz.modular.oawxsendlog.param.OaWxSendLogAddParam;
import vip.xiaonuo.biz.modular.oawxsendlog.param.OaWxSendLogEditParam;
import vip.xiaonuo.biz.modular.oawxsendlog.param.OaWxSendLogIdParam;
import vip.xiaonuo.biz.modular.oawxsendlog.param.OaWxSendLogPageParam;
import vip.xiaonuo.biz.modular.oawxsendlog.service.OaWxSendLogService;

import java.util.List;

/**
 * 微信消息推送日志Service接口实现类
 *
 * @author xiaofan
 * @date  2023/12/18 15:24
 **/
@Service
public class OaWxSendLogServiceImpl extends ServiceImpl<OaWxSendLogMapper, OaWxSendLog> implements OaWxSendLogService {

    @Override
    public Page<OaWxSendLog> page(OaWxSendLogPageParam oaWxSendLogPageParam) {
        QueryWrapper<OaWxSendLog> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(oaWxSendLogPageParam.getSortField(), oaWxSendLogPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(oaWxSendLogPageParam.getSortOrder());
            queryWrapper.orderBy(true, oaWxSendLogPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(oaWxSendLogPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(OaWxSendLog::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(OaWxSendLogAddParam oaWxSendLogAddParam) {
        OaWxSendLog oaWxSendLog = BeanUtil.toBean(oaWxSendLogAddParam, OaWxSendLog.class);
        this.save(oaWxSendLog);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(OaWxSendLogEditParam oaWxSendLogEditParam) {
        OaWxSendLog oaWxSendLog = this.queryEntity(oaWxSendLogEditParam.getId());
        BeanUtil.copyProperties(oaWxSendLogEditParam, oaWxSendLog);
        this.updateById(oaWxSendLog);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<OaWxSendLogIdParam> oaWxSendLogIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(oaWxSendLogIdParamList, OaWxSendLogIdParam::getId));
    }

    @Override
    public OaWxSendLog detail(OaWxSendLogIdParam oaWxSendLogIdParam) {
        return this.queryEntity(oaWxSendLogIdParam.getId());
    }

    @Override
    public OaWxSendLog queryEntity(String id) {
        OaWxSendLog oaWxSendLog = this.getById(id);
        if(ObjectUtil.isEmpty(oaWxSendLog)) {
            throw new CommonException("微信消息推送日志不存在，id值为：{}", id);
        }
        return oaWxSendLog;
    }
}
