/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.openinterface.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.openinterface.entity.BizOpenInterface;
import vip.xiaonuo.biz.modular.openinterface.param.BizOpenInterfaceAddParam;
import vip.xiaonuo.biz.modular.openinterface.param.BizOpenInterfaceEditParam;
import vip.xiaonuo.biz.modular.openinterface.param.BizOpenInterfaceIdParam;
import vip.xiaonuo.biz.modular.openinterface.param.BizOpenInterfacePageParam;
import vip.xiaonuo.biz.modular.openinterface.service.BizOpenInterfaceService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 应用管理控制器
 *
 * @author 徐超
 * @date  2025/04/08 13:25
 */
@Tag(name = "应用管理控制器")
@RestController
@Validated
public class BizOpenInterfaceController {

    @Resource
    private BizOpenInterfaceService bizOpenInterfaceService;

    /**
     * 获取应用管理分页
     *
     * @author 徐超
     * @date  2025/04/08 13:25
     */
    @Operation(summary = "获取应用管理分页")
    @SaCheckPermission("/biz/openinterface/page")
    @GetMapping("/biz/openinterface/page")
    public CommonResult<Page<BizOpenInterface>> page(BizOpenInterfacePageParam bizOpenInterfacePageParam) {
        return CommonResult.data(bizOpenInterfaceService.page(bizOpenInterfacePageParam));
    }

    /**
     * 添加应用管理
     *
     * @author 徐超
     * @date  2025/04/08 13:25
     */
    @Operation(summary = "添加应用管理")
    @CommonLog("添加应用管理")
    @SaCheckPermission("/biz/openinterface/add")
    @PostMapping("/biz/openinterface/add")
    public CommonResult<String> add(@RequestBody @Valid BizOpenInterfaceAddParam bizOpenInterfaceAddParam) {
        bizOpenInterfaceService.add(bizOpenInterfaceAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑应用管理
     *
     * @author 徐超
     * @date  2025/04/08 13:25
     */
    @Operation(summary = "编辑应用管理")
    @CommonLog("编辑应用管理")
    @SaCheckPermission("/biz/openinterface/edit")
    @PostMapping("/biz/openinterface/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizOpenInterfaceEditParam bizOpenInterfaceEditParam) {
        bizOpenInterfaceService.edit(bizOpenInterfaceEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除应用管理
     *
     * @author 徐超
     * @date  2025/04/08 13:25
     */
    @Operation(summary = "删除应用管理")
    @CommonLog("删除应用管理")
    @SaCheckPermission("/biz/openinterface/delete")
    @PostMapping("/biz/openinterface/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizOpenInterfaceIdParam> bizOpenInterfaceIdParamList) {
        bizOpenInterfaceService.delete(bizOpenInterfaceIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取应用管理详情
     *
     * @author 徐超
     * @date  2025/04/08 13:25
     */
    @Operation(summary = "获取应用管理详情")
    @SaCheckPermission("/biz/openinterface/detail")
    @GetMapping("/biz/openinterface/detail")
    public CommonResult<BizOpenInterface> detail(@Valid BizOpenInterfaceIdParam bizOpenInterfaceIdParam) {
        return CommonResult.data(bizOpenInterfaceService.detail(bizOpenInterfaceIdParam));
    }
}
