/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.supplier.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.PhoneUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.supplier.entity.BizSupplier;
import vip.xiaonuo.biz.modular.supplier.mapper.BizSupplierMapper;
import vip.xiaonuo.biz.modular.supplier.param.BizSupplierAddParam;
import vip.xiaonuo.biz.modular.supplier.param.BizSupplierEditParam;
import vip.xiaonuo.biz.modular.supplier.param.BizSupplierIdParam;
import vip.xiaonuo.biz.modular.supplier.param.BizSupplierPageParam;
import vip.xiaonuo.biz.modular.supplier.service.BizSupplierService;

import java.util.List;

/**
 * 供货商Service接口实现类
 *
 * @author fanzherong
 * @date  2025/03/21 10:38
 **/
@Service
public class BizSupplierServiceImpl extends ServiceImpl<BizSupplierMapper, BizSupplier> implements BizSupplierService {

    @Override
    public Page<BizSupplier> page(BizSupplierPageParam bizSupplierPageParam) {
        QueryWrapper<BizSupplier> queryWrapper = new QueryWrapper<BizSupplier>().checkSqlInjection();
        if(ObjectUtil.isNotEmpty(bizSupplierPageParam.getSupplierName())){
            queryWrapper.lambda().like(BizSupplier::getSupplierName,bizSupplierPageParam.getSupplierName());
        }
        if(ObjectUtil.isNotEmpty(bizSupplierPageParam.getSupplierContactName())){
            queryWrapper.lambda().like(BizSupplier::getSupplierContactName,bizSupplierPageParam.getSupplierContactName());
        }
        queryWrapper.lambda().orderByDesc(BizSupplier::getCreateTime);
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BizSupplierAddParam bizSupplierAddParam) {
        //校验手机号格式
        if(ObjectUtil.isNotEmpty(bizSupplierAddParam.getSupplierPhone())){
            if(!PhoneUtil.isMobile(bizSupplierAddParam.getSupplierPhone())) {
                throw new CommonException("手机号码：{}格式错误", bizSupplierAddParam.getSupplierPhone());
            }
        }
        //判断供货商是否存在
        long count = this.count(new QueryWrapper<BizSupplier>().lambda().eq(BizSupplier::getSupplierName, bizSupplierAddParam.getSupplierName()));
        if(count>0){
            throw new CommonException("供货商已存在！");
        }
        BizSupplier bizSupplier = BeanUtil.toBean(bizSupplierAddParam, BizSupplier.class);
        this.save(bizSupplier);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(BizSupplierEditParam bizSupplierEditParam) {
        //校验手机号格式
        if(ObjectUtil.isNotEmpty(bizSupplierEditParam.getSupplierPhone())){
            if(!PhoneUtil.isMobile(bizSupplierEditParam.getSupplierPhone())) {
                throw new CommonException("手机号码：{}格式错误", bizSupplierEditParam.getSupplierPhone());
            }
        }
        BizSupplier bizSupplier = this.queryEntity(bizSupplierEditParam.getId());
        if(!StringUtils.equals(bizSupplier.getSupplierName(),bizSupplierEditParam.getSupplierName())){
            //修改过供货商名称，判断供货商是否存在
            long count = this.count(new QueryWrapper<BizSupplier>().lambda().eq(BizSupplier::getSupplierName, bizSupplierEditParam.getSupplierName()));
            if(count>0){
                throw new CommonException("供货商已存在！");
            }
        }
        BeanUtil.copyProperties(bizSupplierEditParam, bizSupplier);
        this.updateById(bizSupplier);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<BizSupplierIdParam> bizSupplierIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(bizSupplierIdParamList, BizSupplierIdParam::getId));
    }

    @Override
    public BizSupplier detail(BizSupplierIdParam bizSupplierIdParam) {
        return this.queryEntity(bizSupplierIdParam.getId());
    }

    @Override
    public BizSupplier queryEntity(String id) {
        BizSupplier bizSupplier = this.getById(id);
        if(ObjectUtil.isEmpty(bizSupplier)) {
            throw new CommonException("供货商不存在，id值为：{}", id);
        }
        return bizSupplier;
    }
}
